/*
Class:   MG_Mover
Authors:  Gary White and Jean-Pierre Martel

Based on a Gary White's Form published in the dBase Inc's
   "dbase.programming" newsgroup, May 8th 1999,
   in the thread called "Combo Boxes". Few ideas were taken
   from Dan Howard's ItemMover class.

Purpose:
   Use this object to select items in a list. 

This class requires:
   an array called a_Items filled with some items

Methods:

   CountItems: To count the number of items chosen
      and to enable/disable the needed pushbuttons.

   MoveItem: The heart of this custom class. To move items
      from one listbox to the other.

   OnOpen: To populate listbox1 if the needed array doesn't exists.

   Reset: To reset the MG_Mover at its original state.

   Want_All: To select all items.

   The MG_Mover allows multiple selection by default. If you want
   single-click moving, activate the lines presently inactivated in the
   listboxes construction code and block the two lines of code just over
   the lines you revealed. Then use the first version of the MoveItem function
   (instead of the second one).

   When this Custom class is dropped on a form, the Form designer
   will stream a bunch of unnecessary code that can be safely removed:
               with(this.MG_Mover1.P_PLUS)
               value = false
            endwith

The authors accept no responsibility for the use or consequences 
of the use of this software (My lawyer should be proud of me).

*/

CLASS MG_Mover(ParentObj) of CONTAINER(ParentObj) CUSTOM
   with(this)
   onOpen := CLASS::ONOPEN
   parent.metric := 6 // Pixels
   borderStyle := 4	// Single
   width := 474
   height := 171
   anchor := 1 //bottom
   visible := false
   endwith


   this.RECTANGLE1 = new RECTANGLE(this)
   with (this.RECTANGLE1)
      left := 3
      top := 3
      width := 467
      height := 140
      text := ""
      border := false
      borderStyle := 7	// Client
   endwith


   this.LISTBOX1 = new LISTBOX(this)
   with (this.LISTBOX1)
//      onLeftDblClick := {;this.parent.MoveItem(this.parent.listbox1.selected(),true)}
      multiple := true  // take it out if using single-click moving
//      onLeftDblClick := {;this.parent.want_all()}
//      onLeftMouseDown := {;this.parent.MoveItem(this.parent.listbox1.selected(),true)}
      onRightMouseDown := {;this.parent.want_all()}
      dataSource := 'ARRAY {"Item No. 1","Item No. 2","Item No. 3"}'
      height := 109
      left := 7
      top := 28
      width := 213
      id := 101
      fontSize := 9
      borderStyle := 8	// Modal
   endwith


   this.LISTBOX2 = new LISTBOX(this)
   with (this.LISTBOX2)
//      onLeftDblClick := {;this.parent.MoveItem(this.parent.listbox2.selected(),false)}
      multiple := true  // take it out if using single-click moving
//      onLeftDblClick := {;this.parent.reset()}
//      onLeftMouseDown := {;this.parent.MoveItem(this.parent.listbox2.selected(),false)}
      onRightMouseDown := {;this.parent.reset()}
      height := 109
      left := 252
      top := 28
      width := 213
      id := 101
      fontSize := 9
      borderStyle := 8	// Modal
   endwith


   this.T_POSSIBLE = new TEXT(this)
   with (this.T_POSSIBLE)
      height := 21
      left := 7
      top := 7
      width := 213
      colorNormal := "white/black"
      fontSize := 8
      fontBold := true
      text := " List of Possible Items"
      borderStyle := 8	// Modal
   endwith


   this.T_FINAL = new TEXT(this)
   with (this.T_FINAL)
      height := 21
      left := 252
      top := 7
      width := 213
      colorNormal := "white/black"
      fontSize := 8
      fontBold := true
      text := " Final List"
      borderStyle := 8	// Modal
   endwith


   this.P_PLUS = new PUSHBUTTON(this)
   with (this.P_PLUS)
      onClick := {;this.parent.MoveItem(this.parent.listbox1.selected(),true)}
      height := 14
      left := 226
      top := 42
      width := 21
      upBitmap := "RESOURCE #854"
      disabledBitmap := "RESOURCE #864"
      text := ""
      value := false
      speedBar = true
   endwith


   this.P_ALL = new PUSHBUTTON(this)
   with (this.P_ALL)
      onClick := {;this.parent.want_all()}
      height := 14
      left := 226
      top := 59
      width := 21
      upBitmap := "RESOURCE #855"
      disabledBitmap := "RESOURCE #865"
      text := ""
      value := false
      speedBar = true
   endwith


   this.P_MINUS = new PUSHBUTTON(this)
   with (this.P_MINUS)
      onClick := {;this.parent.MoveItem(this.parent.listbox2.selected(),false)}
      height := 14
      left := 226
      top := 80
      width := 21
      upBitmap := "RESOURCE #853"
      disabledBitmap := "RESOURCE #863"
      text := ""
      value := false
      speedBar = true
   endwith


   this.P_NONE = new PUSHBUTTON(this)
   with (this.P_NONE)
      onClick := {;this.parent.reset()}
      height := 14
      left := 226
      top := 98
      width := 21
      upBitmap := "RESOURCE #852"
      disabledBitmap := "RESOURCE #862"
      text := ""
      value := false
      speedBar = true
   endwith


   this.P_OK = new PUSHBUTTON(this)
   with (this.P_OK)
//   	onClick := {;form.close()}
      height := 16
      left := 337
      top := 149
      width := 65
      text := "OK"
      fontSize := 8
      fontBold := true
      value := false
   endwith


   this.P_CANCEL = new PUSHBUTTON(this)
   with (this.P_CANCEL)
      onClick := {;form.close()}
      height := 16
      left := 406
      top := 149
      width := 65
      text := "Cancel"
      fontSize := 8
      fontBold := true
      value := false
   endwith


  this.T_MESSAGE = new TEXT(this)
   with (this.T_MESSAGE)
      height := 14
      left := 3
      top := 149
      width := 217
      fontSize := 8
      borderStyle := 3	// None
   endwith


   function reset
      if this.chosen.size # 0
       	this.available = new array( this.a_Items.size )
         acopy( this.a_Items, this.available )
         this.chosen=new array()
         this.listbox1.datasource := this.listbox1.datasource
         this.listbox2.datasource := this.listbox2.datasource
         this.countItems()
         this.listbox1.cursel := 1
      endif
      return

/*
   function MoveItem( cItem, bAdd )   // if you allow single-clicking to move
      if not empty( cItem)
         oFrom = iif( bAdd, this.listbox1, this.listbox2 )
         oTo = iif( bAdd, this.listbox2, this.listbox1 )
         if bAdd
            this.chosen.add( cItem )
            this.chosen.sort()
            this.available.delete( this.available.scan( cItem ))
            this.available.size --
         else
            this.available.add( cItem )
            this.available.sort()
            this.chosen.delete( this.chosen.scan( cItem ))
            this.chosen.size --
         endif
         oFrom.datasource := oFrom.datasource
         oTo.datasource := oTo.datasource
         oTo.cursel := 1
         do while oTo.value # cItem
            oTo.cursel ++
         enddo
         this.countItems()
      endif
      return
*/

   function MoveItem( aItem, bAdd )  // if you allow multiple selection
      if not empty( aItem)
         oFrom = iif( bAdd, this.listbox1, this.listbox2 )
         oTo = iif( bAdd, this.listbox2, this.listbox1 )
         for i = 1 to aItem.size
            if bAdd
               this.chosen.add( aItem[i] )
               this.chosen.sort()
               this.available.delete( this.available.scan( aItem[i] ))
               this.available.size --
            else
               this.available.add( aItem[i] )
               this.available.sort()
               this.chosen.delete( this.chosen.scan( aItem[i] ))
               this.chosen.size --
            endif
         endfor
         oFrom.datasource := oFrom.datasource
         oTo.datasource := oTo.datasource
         oTo.cursel := 1
         do while oTo.value # aItem[aItem.size]
            oTo.cursel ++
         enddo
         this.countItems()
      endif
      return


   function countItems
      do case
         case this.Listbox2.count() = 0
            this.T_Message.text := "   No item chosen among " + ltrim(str(aLen(this.a_Items,1))) + " possible"
         case this.Listbox2.count() = 1
            this.T_Message.text := "   1 item chosen among " + ltrim(str(aLen(this.a_Items,1))) + " possible"
         otherwise
            this.T_Message.text := "   " + this.Listbox2.count() + " items chosen among " + ltrim(str(aLen(this.a_Items,1))) + " possible"
      endcase
      this.P_Minus.enabled := this.Listbox2.count() > 0
      this.Listbox2.mousePointer := iif(this.P_Minus.enabled = true, 13, 12)
      this.P_None.enabled := this.Listbox2.count() > 0
      this.P_OK.enabled := this.Listbox2.count() > 0
      this.P_Plus.enabled := this.Listbox1.count() > 0
      this.Listbox1.mousePointer := iif(this.P_Plus.enabled = true, 13, 12)
      this.P_All.enabled := this.Listbox1.count() > 0
      return


   function want_All
      if this.available.size # 0
         this.chosen.resize(this.a_Items.size)
         acopy(this.a_Items, this.chosen)
         this.listbox2.datasource := this.listbox2.datasource
         this.available.size := 0
         this.listbox1.datasource := this.listbox1.datasource
         this.countItems()
         this.listbox2.cursel := 1
      endif
      return


   function onOpen
      this.borderStyle := 3
   	if type("this.a_Items") # "A"
         this.a_Items=new array()
         For i = 1 to 9
            this.a_Items.add("Item No. " + i)
         Next
      else
         this.a_Items.sort()
      endif

    	this.available = new array( this.a_Items.size )
      acopy( this.a_Items, this.available )
   	this.listbox1.datasource := 'array this.available'
      this.listbox1.cursel := 1
      this.chosen = new array()
   	this.listbox2.datasource := 'array this.chosen'
      this.countItems()
      this.visible := true
		return

endclass

